/**************************************************************************
 *  file: SYNTAX_CHECKER.c , Check syntax of all files     Part of DIKUMUD *
 *  Usage: QUICK AND DIRTY!!                                               *
 *  Copyright (C) 1990, 1991 - see 'license.doc' for complete information. *
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "structs.h"
#include "utils.h"
#include "db.h"
#include "comm.h"
#include "handler.h"
#include "limits.h"

/**************************************************************************
*  declarations of most of the 'global' variables                         *
************************************************************************ */

struct room_data *world;              /* dyn alloc'ed array of rooms     */
int top_of_world = 0;                 /* ref to the top element of world */
struct obj_data  *object_list = 0;    /* the global linked list of obj's */
struct char_data *character_list = 0; /* global l-list of chars          */

struct zone_data *zone_table;         /* table of reset data             */
int top_of_zone_table = 0;
struct message_list fight_messages[MAX_MESSAGES]; /* fighting messages   */
struct player_index_element *player_table = 0; /* index to player file   */
int top_of_p_table = 0;               /* ref to top of table             */
int top_of_p_file = 0;

FILE *newf;

FILE *mob_f,                          /* file containing mob prototypes  */
     *obj_f,                          /* obj prototypes                  */
     *wld_f,                          /* World file                      */
     *zon_f;

struct index_data *mob_index;         /* index table for mobile file     */
struct index_data *obj_index;         /* index table for object file     */
struct index_data *wld_index;

struct help_index_element *help_index = 0;

int top_of_mobt = 0;                  /* top of mobile index table       */
int top_of_objt = 0;                  /* top of object index table       */
int top_of_wldt = 0;

struct time_data time_info;      /* the infomation about the time   */
struct weather_data weather_info;   /* the infomation about the weather */


/* local procedures */
void boot_zones(void);
void setup_dir(FILE *fl, int room, int dir);
void allocate_room(int new_top);
void boot_world(void);
struct index_data *generate_indices(FILE *fl, int *top);
void build_player_index(void);
void char_to_store(struct char_data *ch, struct char_file_u *st);
void store_to_char(struct char_file_u *st, struct char_data *ch);
int is_empty(int zone_nr);
void reset_zone(int zone);
int file_to_string(char *name, char *buf);
void renum_world(void);
void renum_zone_table(void);
void reset_time(void);
void clear_char(struct char_data *ch);


/*************************************************************************
*  routines for booting the system                                       *
*********************************************************************** */


void assume(int faktisk, int antal, int place, char *errmsg)
{
   if (antal != faktisk) {
      printf("Error has occured at #%d.\n\r", place);
      printf("Message is : %s\n\r", errmsg);
      printf("Actual number read is %d\n\r", faktisk);
      exit();
   }
}

char *split_names(char *names)
{
   static char buf[4096];
	char *b;

   b = buf;

	*b++ = '"';

   for(;*names; names++)
	{
	  if (*names == ' ')
	  {
		 *b++ = '"';
		 *b++ = ',';
		 *b++ = '"';
	  }
	  else
	    *b++ = *names;
	}

	*b++ = '"';
	*b++ = '\0';

	return buf;
}


char *find_position(int pos)
{
	static char *positions[] =
	{	
		"POSITION_DEAD",
		"POSITION_MORTALLYW",
		"POSITION_INCAP",
		"POSITION_STUNNED",
		"POSITION_SLEEPING",
		"POSITION_RESTING",
		"POSITION_SITTING",
		"POSITION_FIGHTING",
		"POSITION_STANDING"
	};

   if ((pos >= POSITION_DEAD) && (pos <= POSITION_STANDING))
		return positions[pos];
	else
		return positions[POSITION_STANDING];
}

char *find_exit_flags(int flags)
{
	static char buf[256];
	int used;

	buf[0] = '{';
	buf[1] = '\0';

	used = FALSE;
	if (IS_SET(flags, 1))
	{
		used = TRUE;
		strcat(buf, "EX_OPEN_CLOSE");
	}

	if (IS_SET(flags, 2))
   {
		if (used)
			strcat(buf, ", EX_CLOSED");
		else
			strcat(buf, "EX_CLOSED");
		used = TRUE;
	}
	if (IS_SET(flags, 4))
   {
		if (used)
			strcat(buf, ", EX_LOCKED");
		else
			strcat(buf, "EX_LOCKED");
		used = TRUE;
	}
	if (IS_SET(flags, 32))
   {
		if (used)
			strcat(buf, ", EX_PICKPROOF");
		else
			strcat(buf, "EX_PICKPROOF");
		used = TRUE;
	}

	strcat(buf, "}");

	return buf;
}

char *find_eqp_pos(int pos)
{
	static char *eqp_pos[] =
	{	
		"XXX WEAR_ILLEGAL! (WAS LIGHT, NOW HOLD)",
		"WEAR_FINGER_R",
		"WEAR_FINGER_L",
		"WEAR_NECK_1",
		"WEAR_NECK_2",
		"WEAR_BODY",
		"WEAR_HEAD",
		"WEAR_LEGS",
		"WEAR_FEET",
		"WEAR_HANDS",
		"WEAR_ARMS",
		"WEAR_SHIELD",
		"WEAR_ABOUT",
		"WEAR_WAISTE",
		"WEAR_WRIST_R",
		"WEAR_WRIST_L",
		"WEAR_WIELD",
		"WEAR_HOLD"
	};

   if ((pos >= 0) && (pos <= 17))
		return eqp_pos[pos];
	else
		return eqp_pos[0];
}

char *find_direction(int dir)
{
   static char *exits[] =
   {
      "NORTH",
      "EAST",
      "SOUTH",
      "WEST",
      "UP",
      "DOWN" };

   if ((dir < 0) || (dir > 5))
      return "";
   else
      return exits[dir];
}

char *find_direction2(int dir)
{
   static char *exits[] =
   {
      "north",
      "east",
      "south",
      "west",
      "up",
      "down" };

   if ((dir < 0) || (dir > 5))
      return "";
   else
      return exits[dir];
}


/* generate index table for object, monster or world file*/
struct index_data *generate_indices(FILE *fl, int *top)
{
   int i = 0, antal;
   struct index_data *index;
   long pos;
   char buf[82];

   rewind(fl);

   for (;;)
   {
      if (fgets(buf, 81, fl))
      {
         if (*buf == '#')
         {
            /* allocate new cell */
            
            if (!i)                   /* first cell */
               CREATE(index, struct index_data, 1);
            else
               if (!(index = 
                  (struct index_data*) realloc(index, 
                  (i + 1) * sizeof(struct index_data))))
               {
                  printf("load indices");
                  exit();
                }
            antal = sscanf(buf, "#%d", &index[i].virtual);
            assume(antal, 1, index[i].virtual, "Next string with E/A/$");

            index[i].pos = ftell(fl);
            index[i].number = index[i].virtual;
            index[i].func = 0;
            i++;
         }
         else 
            if (*buf == '$')   /* EOF */
               break;
      }
      else
      {
         printf("Error when generating index, based upon #xxxx numbers.\n\r");
         printf("   Probably error at end of file.\n\r");

         exit();
      }
   }
   index[i-1].number = -1;
   *top = i - 1;
   return(index);
}

int exist_index(struct index_data *index_list, int top, int num)
{
   int i, found;

   found = FALSE;

   for(i=0; (i<=top) && !(found); i++)
      if (index_list[i].number == num)
         found = TRUE;

   if (!found) {
      printf("Reference to non-existent number #%d\n\r", num);
   }

   return (found);
}


/* check the rooms */
void check_world(FILE *fl)
{
   int room_nr = 0, zone = 0, dir_nr, virtual_nr, flag, tmp, old_virtual;
   char *temp, chk[50], buf[500];
   struct extra_descr_data *new_descr;

   int antal;
	int movement, flags;

   world = 0;
   character_list = 0;
   object_list = 0;

   rewind(fl);

   old_virtual = -1;

   fprintf(newf, "%%rooms\n\n");

   do
   {
      antal = fscanf(fl, " #%d\n", &virtual_nr);
      assume(antal,1, virtual_nr, "Reading #xxx");

      if (old_virtual > virtual_nr)
         assume(0,1,virtual_nr, "Error - #'s not in order.");
      old_virtual = virtual_nr;

      fprintf(newf, "\n                rn_%d\n\n", virtual_nr);
      temp = fread_string(fl);
      if (flag = (*temp != '$'))   /* a new record to be read */
      {
         fprintf(newf, "title \"%s\"\n", temp); /* Title */

         temp = fread_string(fl);
         fprintf(newf, "descr\n\"%s\"\n",temp); /* Description */

         antal = fscanf(fl, " %d ", &tmp);
         assume(antal,1, virtual_nr, "In room basic 3 numbers");
         /* print nothing about zone_nr */

         antal = fscanf(fl, " %d ", &flags);
         assume(antal,1, virtual_nr, "In room basic 3 numbers");
			/* Written after rest */

         antal = fscanf(fl, " %d ", &movement);
         assume(antal,1, virtual_nr, "In room basic 3 numbers");
			/* Written after directions */

         for (;;)
         {
            antal = fscanf(fl, " %s \n", chk);
            assume(antal,1, virtual_nr, "Reading D/E/S string");

            if (*chk == 'D')  /* direction field */
               setup_dir(fl, virtual_nr, atoi(chk + 1));
            else if (*chk == 'E')  /* extra description field */
            {
               temp = fread_string(fl); /* Keywords    */
               fprintf(newf, "extra {%s}", split_names(temp));
               temp = fread_string(fl); /* Description */
               fprintf(newf, "\n\"%s\"\n", temp);
            }
            else if (*chk == 'S')   /* end of current room */
               break;
            else
               assume(FALSE, 0, virtual_nr, "MISSING D/E or S");
			}

         fprintf(newf, "\nmovement ");
         switch (movement)
         {
            case SECT_INSIDE:
               fprintf(newf, "SECT_INSIDE\n");
               break;
            case SECT_CITY:
               fprintf(newf, "SECT_CITY\n");
               break;
            case SECT_FIELD:
               fprintf(newf, "SECT_FIELD\n");
               break;
            case SECT_FOREST:
               fprintf(newf, "SECT_FOREST\n");
               break;
            case SECT_HILLS:
               fprintf(newf, "SECT_HILLS\n");
               break;
            case SECT_MOUNTAIN:
               fprintf(newf, "SECT_MOUNTAIN\n");
               break;
            case SECT_WATER_SWIM:
               fprintf(newf, "SECT_LOW_WATER\n");
               break;
            case SECT_WATER_NOSWIM:
               fprintf(newf, "SECT_DEEP_WATER\n");
               break;
            default:
               fprintf(newf, "WHAT???\n");
               break;
         }

         if (flags)
         {
            temp = buf;
            *temp = '\0';

            if (IS_SET(flags, NO_MOB))
            {
               if (*temp)
                  strcat(temp, ", UNIT_FL_NO_MOB");
               else
                  strcpy(temp, "UNIT_FL_NO_MOB");
            }

            if   (IS_SET(flags, INDOORS))
            {
               if (*temp)
                  strcat(temp, ", UNIT_FL_INDOORS");
               else
                  strcpy(temp, "UNIT_FL_INDOORS");
            }

            if (IS_SET(flags, PRIVATE))
            {
               if (*temp)
                  strcat(temp, ", UNIT_FL_PRIVATE");
               else
                  strcpy(temp, "UNIT_FL_PRIVATE");
            }

            if (*buf)
               fprintf(newf, "flags {%s}\n", buf);
         }

			fprintf(newf, "end\n\n");
      }
   }
   while (flag);
}




/* read direction data */
void setup_dir(FILE *fl, int room, int dir)
{
   int tmp1, tmp2, tmp3, antal;
   char *temp;
   char buf[MAX_STRING_LENGTH];

   strcpy(buf, fread_string(fl)); /* General descr */
   temp = fread_string(fl); /* Keyword List  */

   antal = fscanf(fl, " %d ", &tmp1);
   assume(antal,1, room, "One of three Direction data");
   antal = fscanf(fl, " %d ", &tmp2);
   assume(antal,1, room, "One of three Direction data");
   antal = fscanf(fl, " %d ", &tmp3);
   assume(antal,1, room, "One of three Direction data");
   exist_index(wld_index, top_of_wldt, tmp3);

   if (tmp3 >= 0)
   {
      fprintf(newf, "%s to rn_%d",
              find_direction2(dir),
              tmp3);

      if (*buf)
         fprintf(newf, " descr\n\"%s\"", buf);

      if (temp && *temp)
         fprintf(newf, " keyword {\"%s\"}", temp);

      fprintf(newf, ";\n");
   }
}


#define NEW_ZONE_SYSTEM xxx

/* load the zone table and command tables */
void check_zones(FILE *fl)
{
   int line_no;
   int antal, tmp1, tmp2, tmp3, tmp4;
   int zon = 0, cmd_no = 0, ch, expand;
   char *check, buf[81];
   char cmd_type;
	int nest, prenest;

   rewind(fl);
   line_no=1;

   fprintf(newf, "%%reset\n\n");

   for (;;)
   {
      antal = fscanf(fl, " #%*d\n");
      assume(antal, 0, line_no++, "Zone number not found");

      check = fread_string(fl);
      line_no++;

      if (*check == '$')
         break;      /* end of file */

      /* alloc a new zone */

#ifdef NEW_ZONE_SYSTEM
      antal = fscanf(fl, " %d ", &zon);
      assume(antal, 1, line_no, "Zone Room < number not found");
#endif

      antal = fscanf(fl, " %d ", &zon);
      assume(antal, 1, line_no, "Life Span");

      antal = fscanf(fl, " %d ", &zon);
      assume(antal, 1, line_no++, "Reset Mode");


      /* read the command table */

      cmd_no = 0;
		nest = prenest = 0;

      for (expand = 1;;)
      {

         fscanf(fl, " "); /* skip blanks */
         antal = fscanf(fl, "%c", &cmd_type);
         assume(antal, 1, line_no, "Command type M/*/O/G/E/S missing");
         
         if (cmd_type == 'S')
            break;

         if (cmd_type == '*')
         {
            expand = 0;
            fgets(buf, 80, fl); /* skip command */
            line_no++;
            continue;
         }

         antal = fscanf(fl, " %d %d %d", &tmp1, &tmp2, &tmp3);
         assume(antal, 3, line_no, "Three values after command missing");

         if (cmd_type == 'M' || cmd_type == 'O' ||
             cmd_type == 'D' || cmd_type == 'P' || cmd_type == 'E') {
            antal = fscanf(fl, " %d", &tmp4);
            assume(antal, 1, line_no, "Fourth value after command missing");
         }

			if (!nest && tmp1)
			{
				fprintf(newf, "{\n");
				nest = 1;
			}

			if (nest && !tmp1)
			{
				fprintf(newf, "}\n");
				nest = 0;
			}
			if (tmp1)
			   fprintf(newf, "    ");

         switch (cmd_type)
			{
            case 'M' :
               exist_index(mob_index, top_of_mobt, tmp2);
               exist_index(wld_index, top_of_wldt, tmp4);
               fprintf(newf, "load mn_%d into rn_%d max %d\n",
                       tmp2, tmp4, tmp3);
               break;
            case 'O' :
               exist_index(obj_index, top_of_objt, tmp2);
               exist_index(wld_index, top_of_wldt, tmp4);
               fprintf(newf, "load on_%d into rn_%d max %d\n",
                       tmp2, tmp4, tmp3);
               break;
            case 'G' : /* Must be nested! */
               exist_index(obj_index, top_of_objt, tmp2);
               fprintf(newf, "load on_%d max %d\n",
                       tmp2, tmp3);
               break;
            case 'E' :
               exist_index(obj_index, top_of_objt, tmp2);
               fprintf(newf, "equip on_%d max %d position %s\n",
                       tmp2, tmp3, find_eqp_pos(tmp4));
               break;
            case 'P' :
               exist_index(obj_index, top_of_objt, tmp2);
               exist_index(obj_index, top_of_objt, tmp4);
               fprintf(newf, "load on_%d max %d\n",
                       tmp2, tmp3);
               break;
            case 'D' :
               exist_index(wld_index, top_of_wldt, tmp2);
               fprintf(newf, "door rn_%d %s %s\n",
                       tmp2, find_direction(tmp3), find_exit_flags(tmp4));
               break;
            case 'R' :
               exist_index(wld_index, top_of_wldt, tmp2);
               exist_index(obj_index, top_of_objt, tmp3);
               fprintf(newf, "remove on_%d in rn_%d\n",
                       tmp3, tmp2);
               break;
            case '*' :
               break;
            deafult  :
               printf("Illegal command type");
               exit();
               break;
         }

         fgets(buf, 80, fl);   /* read comment */
         line_no++;
      }
   }
}
 




/*************************************************************************
*  procedures for resetting, both play-time and boot-time        *
*********************************************************************** */


void write_position(int tmp)
{
   switch (tmp)
   {
      case POSITION_STANDING:
         fprintf(newf, "POSITION_STANDING\n");
         break;
      case POSITION_SITTING:
         fprintf(newf, "POSITION_SITTING\n");
         break;
      case POSITION_RESTING:
         fprintf(newf, "POSITION_RESTING\n");
         break;
      case POSITION_SLEEPING:
         fprintf(newf, "POSITION_SLEEPING\n");
         break;
   }
}


/* read a mobile from MOB_FILE */
void check_mobile(FILE *fl)
{
   int virtual_nr, old_virtual, antal, flag;
   char *temp;
   char bogst;
   char unit_fl[500], char_fl[500], npc_fl[500],
	     special[4096];

   int i, skill_nr;
   long tmp, tmp2, tmp3;
   struct char_data *mob;
   char chk[10];

   old_virtual = -1;
   rewind(fl);

   fprintf(newf, "%%mobiles\n\n");

   do
   {
      antal = fscanf(fl, " #%d\n", &virtual_nr);
      assume(antal,1, virtual_nr, "Reading #xxx");
      if (old_virtual > virtual_nr)
         assume(0,1,virtual_nr, "Error - #'s not in order.");

      old_virtual = virtual_nr;


      temp = fread_string(fl);  /* Namelist */
      if (flag = (*temp != '$'))
      {   /* a new record to be read */
         fprintf(newf, "\n\n                mn_%d\n", virtual_nr);
         fprintf(newf, "names {%s}\n", split_names(temp));

         /***** String data *** */

         /* Name already read mob->player.name = fread_string(fl); */
         temp = fread_string(fl);  /* short description  */
         fprintf(newf, "title \"%s\"\n",temp);

         temp = fread_string(fl);  /* long_description    */
         fprintf(newf, "descr \"%s\"\n", temp);

         temp = fread_string(fl);  /* look at description */
         fprintf(newf, "extra {} \"%s\"\n", temp);

         /* *** Numeric data *** */

         antal = fscanf(fl, "%d ", &tmp);
         assume(antal, 1, virtual_nr, "ACT error");
         *unit_fl = '\0';
         *char_fl = '\0';
         *npc_fl = '\0';
			*special = '\0';

			if (tmp)
         {
            if (!IS_SET(tmp, ACT_SENTINEL))
            {
					if (IS_SET(tmp, ACT_STAY_ZONE))
						strcat(special, "special SFUN_RANDOM_ZONE_MOVE time 40 bits SFB_TIME1\n");
					else
						strcat(special, "special SFUN_RANDOM_MOVE time 40 bits SFB_TIME1\n");
            }

            if (IS_SET(tmp, ACT_SCAVENGER))
            {
					strcat(special, "special SFUN_SCAVENGER time 40 bits SFB_TIME1\n");
            }

            if (IS_SET(tmp, ACT_NICE_THIEF))
            {
					/* Non-existant */
            }

            if (IS_SET(tmp, ACT_AGGRESSIVE))
            {
					strcat(special, "special SFUN_AGGRESSIVE time 40\n");
            }

            if (IS_SET(tmp, ACT_WIMPY))
            {
               if (*char_fl)
                  strcat(char_fl, ", CHAR_WIMPY");
               else
                  strcat(char_fl, "CHAR_WIMPY");
            }
         }
		
         antal = fscanf(fl, " %d ", &tmp);
         assume(antal, 1, virtual_nr, "affected_by error");
         if (tmp)
         {
            if (IS_SET(tmp, AFF_BLIND))
            {
               if (*char_fl)
                  strcat(char_fl, ", CHAR_BLIND");
               else
                  strcat(char_fl, "CHAR_BLIND");
            }

            if (IS_SET(tmp, AFF_INVISIBLE))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", UNIT_FL_INVISIBLE");
               else
                  strcat(unit_fl, "UNIT_FL_INVISIBLE");
            }

            if (IS_SET(tmp, AFF_DETECT_EVIL))
            {
               if (*char_fl)
                  strcat(char_fl, ", CHAR_DETECT_ALIGN");
               else
                  strcat(char_fl, "CHAR_DETECT_ALIGN");
            }

            if (IS_SET(tmp, AFF_DETECT_INVISIBLE))
            {
               if (*char_fl)
                  strcat(char_fl, ", CHAR_DETECT_INVISIBLE");
               else
                  strcat(char_fl, "CHAR_DETECT_INVISIBLE");
            }

            if (IS_SET(tmp, AFF_DETECT_MAGIC))
            {
               if (*char_fl)
                  strcat(char_fl, ", CHAR_DETECT_MAGIC");
               else
                  strcat(char_fl, "CHAR_DETECT_MAGIC");
            }

            if (IS_SET(tmp, AFF_SENSE_LIFE))
            {
               if (*char_fl)
                  strcat(char_fl, ", CHAR_DETECT_LIFE");
               else
                  strcat(char_fl, "CHAR_DETECT_LIFE");
            }

            if (IS_SET(tmp, AFF_HOLD))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_SANCTUARY))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_GROUP))
            {
               if (*char_fl)
                  strcat(char_fl, ", CHAR_GROUP");
               else
                  strcat(char_fl, "CHAR_GROUP");
            }

            if (IS_SET(tmp, AFF_CURSE))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_FLAMING))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_POISON))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_PROTECT_EVIL))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_PARALYSIS))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_MORDEN_SWORD))
            {
               /* Not used */
            }
			
            if (IS_SET(tmp, AFF_FLAMING_SWORD))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_SLEEP))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_DODGE))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_SNEAK))
            {
               if (*char_fl)
                  strcat(char_fl, ", CHAR_SNEAK");
               else
                  strcat(char_fl, "CHAR_SNEAK");
            }

            if (IS_SET(tmp, AFF_HIDE))
            {
               if (*char_fl)
                  strcat(char_fl, ", CHAR_HIDE");
               else
                  strcat(npc_fl, "CHAR_HIDE");
            }

            if (IS_SET(tmp, AFF_FEAR))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_CHARM))
            {
               /* Not used */
            }

            if (IS_SET(tmp, AFF_FOLLOW))
            {
               /* Not used */
            }
         }
		
         if (*unit_fl)
            fprintf(newf, "flags {%s}\n", unit_fl);
         if (*char_fl)
            fprintf(newf, "romflags {%s}\n", char_fl);
         if (*npc_fl)
            fprintf(newf, "npcflags {%s}\n", npc_fl);

         antal = fscanf(fl, " %d ", &tmp);
         assume(antal, 1, virtual_nr, "Monster Alignment Error");
         if (tmp)
            fprintf(newf, "alignment %d\n", tmp);

         antal = fscanf(fl, " %c \n", &bogst);
         assume(antal, 1, virtual_nr, "Simple/Detailed error");

         if (bogst!='S')
            printf("%c %d\n", bogst, bogst);
		
         if (bogst == 'S') {
            /* The new easy monsters */

            antal = fscanf(fl, " %D ", &tmp);
            assume(antal, 1, virtual_nr, "Level error");
            fprintf(newf, "level %d\n", tmp);

            antal = fscanf(fl, " %D ", &tmp);
            assume(antal, 1, virtual_nr, "THAC0 error");
            /* NO THAC0, CONSIDER WEAPONS/ABILITIES */
      
            antal = fscanf(fl, " %D ", &tmp);
            assume(antal, 1, virtual_nr, "AC error");
            /* Adjust tgh */

            antal = fscanf(fl, " %Dd%D+%D ", &tmp, &tmp2, &tmp3);
            assume(antal, 3, virtual_nr, "Hitpoints");
            /* Set hitpoints from abilities */

            antal = fscanf(fl, " %Dd%D+%D \n", &tmp, &tmp2, &tmp3);
            assume(antal, 3, virtual_nr, "Damage error");
	
            antal = fscanf(fl, " %D ", &tmp);
            assume(antal, 1, virtual_nr, "GOLD error");
				if (tmp > 0)
					fprintf(newf, "gold %d\n", tmp);

            antal = fscanf(fl, " %D \n", &tmp);
            assume(antal, 1, virtual_nr, "XP error");
            /* Not Used */

            antal = fscanf(fl, " %D ", &tmp);
            assume(antal, 1, virtual_nr, "POSITION error");

            antal = fscanf(fl, " %D ", &tmp2);
            assume(antal, 1, virtual_nr, "DEFAULT POS error");

				if (tmp2 != POSITION_STANDING)
				{
					fprintf(newf, "default ");
					write_position(tmp2);
				}

				if (tmp2 != tmp)
				{
					fprintf(newf, "position ");
					write_position(tmp);
				}

            antal = fscanf(fl, " %D \n", &tmp);
            assume(antal, 1, virtual_nr, "SEXY error");
            switch (tmp)
            {
               case SEX_NEUTRAL:
                  fprintf(newf, "sex SEX_NEUTRAL\n");
                  break;
               case SEX_MALE:
                  fprintf(newf, "sex SEX_MALE\n");
                  break;
               case SEX_FEMALE:
                  fprintf(newf, "sex SEX_FEMALE\n");
                  break;
            }
				fprintf(newf, "XXX monster abilities\n");
				if (*special)
					fprintf(newf, special);

			} else {  /* The old monsters are down below here */
				/* SECTION DELETED */
			}
			fprintf(newf, "end\n\n");
		}
   }
   while (flag);
}




/* read an object from OBJ_FILE */
void check_objects(FILE *fl)
{
   int virtual_nr, old_virtual, antal, flag, item_type;
	int data0, data1, data2, data3;
   char *temp, unit_fl[500];

   struct obj_data *obj;
   int tmp, i;
   char chk[256];
   struct extra_descr_data *new_descr;

   old_virtual = -1;

   rewind(fl);

   fprintf(newf, "\n\n\n%%objects\n\n");
   antal = fscanf(fl, " %s \n", chk);
   assume(antal, 1, virtual_nr, "First #xxx number");

   do {
      antal = sscanf(chk, " #%d\n", &virtual_nr);
      assume(antal,1, virtual_nr, "Reading #xxx");

      if (old_virtual > virtual_nr)
         assume(0,1,virtual_nr, "Error - #'s not in order.");

      old_virtual = virtual_nr;

      temp = fread_string(fl);  /* Namelist */
      if (flag = (*temp != '$')) {   /* a new record to be read */

         fprintf(newf, "\n                           on_%d\n\n", virtual_nr);
         fprintf(newf, "names {%s}\n", split_names(temp));

         /* *** string data *** */

         temp = fread_string(fl); /* short */
         fprintf(newf, "title \"%s\"\n", temp);

         temp = fread_string(fl); /* descr */
         fprintf(newf, "descr \"%s\"\n", temp);

         temp = fread_string(fl); /* action */
			if (temp && *temp)
				fprintf(newf, "action \"%s\"\n", temp);
                                                

         /* *** numeric data *** */
         antal = fscanf(fl, " %d ", &item_type);
         assume(antal, 1, virtual_nr, "Error reading type flag");

         antal = fscanf(fl, " %d ", &tmp);
         assume(antal, 1, virtual_nr, "Extra Flag");

         *unit_fl = '\0';

         if (tmp)
         {
            if (IS_SET(tmp, ITEM_GLOW))
            {
                /* Not USed */
            }
            if (IS_SET(tmp, ITEM_HUM))
            {
                /* Not USed */
            }
            if (IS_SET(tmp, ITEM_DARK))
            {
                /* Not USed */
            }
            if (IS_SET(tmp, ITEM_LOCK))
            {
                /* Not USed */
            }
            if (IS_SET(tmp, ITEM_EVIL))
               fprintf(newf, "alignment -1000\n");

            if (IS_SET(tmp, ITEM_INVISIBLE))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", UNIT_FL_INVISIBLE\n");
               else
                  strcat(unit_fl, "UNIT_FL_INVISIBLE\n");
           }

            if (IS_SET(tmp, ITEM_MAGIC))
            {
                /* Not USed */
            }

            if (IS_SET(tmp, ITEM_NODROP))
               fprintf(newf, "/* Maybe insert curse affect here? */\n");

            if (IS_SET(tmp, ITEM_BLESS))
               fprintf(newf, "/* Maybe insert bless affect here? */\n");

            if (IS_SET(tmp, ITEM_ANTI_GOOD))
               fprintf(newf, "/* Maybe insert anti-alignment here? */\n");

            if (IS_SET(tmp, ITEM_ANTI_EVIL))
               fprintf(newf, "/* Maybe insert anti-alignment here? */\n");

            if (IS_SET(tmp, ITEM_ANTI_NEUTRAL))
               fprintf(newf, "/* Maybe insert anti-alignment here? */\n");

            if (*unit_fl)
               fprintf(newf, "unitfl %s\n", unit_fl);
         }

         antal = fscanf(fl, " %d ", &tmp);
         assume(antal, 1, virtual_nr, "wear_flags");
         *unit_fl = '\0';
         if (tmp)
         {
            if (IS_SET(tmp, ITEM_TAKE))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_TAKE");
               else
                  strcat(unit_fl, "MANIPULATE_TAKE");
            }
            if (IS_SET(tmp, ITEM_WEAR_FINGER))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_FINGER");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_FINGER");
            }
            if (IS_SET(tmp, ITEM_WEAR_NECK))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_NECK");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_NECK");
            }
            if (IS_SET(tmp, ITEM_WEAR_BODY))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_BODY");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_BODY");
            }
            if (IS_SET(tmp, ITEM_WEAR_HEAD))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_HEAD");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_HEAD");
            }
            if (IS_SET(tmp, ITEM_WEAR_LEGS))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_LEGS");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_LEGS");
            }
            if (IS_SET(tmp, ITEM_WEAR_FEET))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_FEET");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_FEET");
            }
            if (IS_SET(tmp, ITEM_WEAR_HANDS))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_HANDS");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_HANDS");
            }
            if (IS_SET(tmp, ITEM_WEAR_ARMS))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_ARMS");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_ARMS");
            }
            if (IS_SET(tmp, ITEM_WEAR_SHIELD))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_SHIELD");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_SHIELD");
            }
            if (IS_SET(tmp, ITEM_WEAR_ABOUT))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_ABOUT");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_ABOUT");
            }
            if (IS_SET(tmp, ITEM_WEAR_WAISTE))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_WAISTE");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_WAISTE");
            }
            if (IS_SET(tmp, ITEM_WEAR_WRIST))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WEAR_WRIST");
               else
                  strcat(unit_fl, "MANIPULATE_WEAR_WRIST");
            }
            if (IS_SET(tmp, ITEM_WIELD))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_WIELD");
               else
                  strcat(unit_fl, "MANIPULATE_WIELD");
            }
            if (IS_SET(tmp, ITEM_HOLD))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_HOLD");
               else
                  strcat(unit_fl, "MANIPULATE_HOLD");
            }
            if (IS_SET(tmp, ITEM_THROW))
            {
               if (*unit_fl)
                  strcat(unit_fl, ", MANIPULATE_ENTER");
               else
                  strcat(unit_fl, "MANIPULATE_ENTER");
            }

            if (*unit_fl)
               fprintf(newf, "manipulate {%s}\n", unit_fl);
         }

         antal = fscanf(fl, " %d ", &data0);
         assume(antal, 1, virtual_nr, "value[0]");

         antal = fscanf(fl, " %d ", &data1);
         assume(antal, 1, virtual_nr, "value[1]");

         antal = fscanf(fl, " %d ", &data2);
         assume(antal, 1, virtual_nr, "value[2]");

         antal = fscanf(fl, " %d ", &data3);
         assume(antal, 1, virtual_nr, "value[3]");

         if (item_type == ITEM_LIGHT)
            fprintf(newf, "LIGHT_DEF(%d,%d)\n", data2, 1);
         else if (item_type == ITEM_SCROLL)
            fprintf(newf, "type ITEM_SCROLL\n");
         else if (item_type == ITEM_WAND)
            fprintf(newf, "type ITEM_WAND\n");
         else if (item_type == ITEM_STAFF)
            fprintf(newf, "type ITEM_STAFF\n");
         else if (item_type == ITEM_WEAPON)
            fprintf(newf, "WEAPON_DEF(WPN_XXX,,)\n");
         else if (item_type == ITEM_FIREWEAPON)
            fprintf(newf, "type ITEM_FIREWEAPON\n");
         else if (item_type == ITEM_MISSILE)
            fprintf(newf, "type ITEM_MISSILE\n");
         else if (item_type == ITEM_TREASURE)
            fprintf(newf, "ITEM_TREASURE\n");
         else if (item_type == ITEM_ARMOR)
            fprintf(newf, "ARMOUR_DEF(ARM_XXX,,)\n");
         else if (item_type == ITEM_POTION)
            fprintf(newf, "type ITEM_POTION\n");
         else if (item_type == ITEM_WORN)
            fprintf(newf, "type ITEM_WORN\n");
         else if (item_type == ITEM_OTHER)
            fprintf(newf, "type ITEM_OTHER\n");
         else if (item_type == ITEM_TRASH)
            fprintf(newf, "type ITEM_TRASH\n");
         else if (item_type == ITEM_TRAP)
            fprintf(newf, "type ITEM_TRAP\n");
         else if (item_type == ITEM_CONTAINER)
            fprintf(newf, "CONTAINER_DEF(%d)\n", data0);
         else if (item_type == ITEM_NOTE)
            fprintf(newf, "type ITEM_NOTE\n");
         else if (item_type == ITEM_DRINKCON)
            fprintf(newf, "LIQ_DEF(LIQ_%dXXX, %d, %d, %d)\n",
						  data2,data0,data1,data3);
         else if (item_type == ITEM_KEY)
            fprintf(newf, "type ITEM_KEY\n");
         else if (item_type == ITEM_FOOD)
            fprintf(newf, "FOOD_DEF(%d, %d)\n", data0, data3);
         else if (item_type == ITEM_MONEY)
            fprintf(newf, "MONEY_DEF(%d)\n", data0);
         else if (item_type == ITEM_PEN)
            fprintf(newf, "type ITEM_PEN\n");
         else if (item_type == ITEM_BOAT)
            fprintf(newf, "type ITEM_BOAT\n");
         else
            fprintf(newf, "type ITEM_XXX\n");


         antal = fscanf(fl, " %d ", &tmp);
         assume(antal, 1, virtual_nr, "Weight");
         fprintf(newf, "weight %d\n", tmp);

         antal = fscanf(fl, " %d \n", &tmp);
         assume(antal, 1, virtual_nr, "Cost");
         fprintf(newf, "cost %d\n", tmp);

         antal = fscanf(fl, " %d \n", &tmp);
         assume(antal, 1, virtual_nr, "Cost Per Day");
         fprintf(newf, "rent %d\n", tmp);

         /* *** extra descriptions *** */

         while (fscanf(fl, " %s \n", chk), *chk == 'E')
         {
            temp = fread_string(fl);
            fprintf(newf, "extra {%s}\n", split_names(temp));
            temp = fread_string(fl);
            fprintf(newf, "\"%s\"\n", temp);
         }

         for( i = 0 ; (i < MAX_OBJ_AFFECT) && (*chk == 'A') ; i++)
         {
            fprintf(newf, "/* XXX New affect system, old affect dropped */\n");
            antal = fscanf(fl, " %d ", &tmp);
            assume(antal, 1, virtual_nr, "affected location");

            antal = fscanf(fl, " %d \n", &tmp);
            assume(antal, 1, virtual_nr, "Modifier");

            antal = fscanf(fl, " %s \n", chk);
            assume(antal, 1, virtual_nr, "Next string with E/A/$");

         }
			fprintf(newf, "end\n");
      }
   }
   while (flag);
}




/************************************************************************
*  procs of a (more or less) general utility nature                     *
********************************************************************** */


/* read and allocate space for a '~'-terminated string from a given file */
char *fread_string(FILE *fl)
{
   static char buf[MAX_STRING_LENGTH], tmp[100];
   char *rslt;
   register char *point;
   int flag;

   bzero(buf, MAX_STRING_LENGTH);

   do
   {
      if (!fgets(tmp, MAX_STRING_LENGTH, fl))
      {
         printf("fread_str");
         exit();
      }

      if (strlen(tmp) + strlen(buf) > MAX_STRING_LENGTH)
      {
         printf("fread_string: string too large (db.c, fread_string)");
         exit();
      }
      else
         strcat(buf, tmp);

      for (point = buf + strlen(buf) - 2; point >= buf && isspace(*point);
         point--);      
      if (flag = (*point == '~'))
         if (*(buf + strlen(buf) - 3) == '\n')
         {
				*(buf + strlen(buf) - 3) = '\0';
         }
         else
            *(buf + strlen(buf) -2) = '\0';
      else		
      {
			if (*point == '"')
			   *point = '\'';
         *(buf + strlen(buf) + 1) = '\0';
/*         *(buf + strlen(buf)) = '\0';*/
      }
   }
   while (!flag);

   return(buf);
}


int main(int argc, char *argv[])
{

   char name[256];

  if (argc != 2) {
      printf("Usage : syntax_check <BaseFileName>\n\r");
      exit(0);
   }

   strcpy(name, argv[1]);
   strcat(name, ".wld");

   if (!(wld_f = fopen(name, "r")))
   {
      printf("Could not open world file.\n\r");
      exit();
   }
   strcpy(name, argv[1]);
   strcat(name, ".mob");
   if (!(mob_f = fopen(name, "r")))
   {
      printf("Could not open mobile file.\n\r");
      exit();
   }
   strcpy(name, argv[1]);
   strcat(name, ".obj");
   if (!(obj_f = fopen(name, "r")))
   {
      printf("Could not open object file.\n\r");
      exit();
   }
   strcpy(name, argv[1]);
   strcat(name, ".zon");
   if (!(zon_f = fopen(name, "r")))
   {
      printf("Could not open zone file.\n\r");
      exit();
   }

   if (!(newf = fopen("new.zon", "w")))
   {
      printf("Could not open output file.\n\r");
      exit();
   }

   fprintf(newf, "/* CONVERSION OF OLD DIKUMUD FILES */\n");
   fprintf(newf, "\n#include <values.h>\n");

   fprintf(newf, "%%zone         /* ENTER ZONE NAME HERE */\n\n");
   fprintf(newf, "lifespan 60\n");
   fprintf(newf, "reset RESET_IFEMPTY\n\n\n");

   printf("Generating world file indexes.\n\r");
   wld_index = generate_indices(wld_f, &top_of_wldt);

   printf("Generating mobile file indexes.\n\r");
   mob_index = generate_indices(mob_f, &top_of_mobt);

   printf("Generating object file indexes.\n\r");
   obj_index = generate_indices(obj_f, &top_of_objt);

   printf("Checking World File\n\r");
   check_world(wld_f);

   printf("Checking Mobile File (only simple mobiles).\n\r");
   check_mobile(mob_f);

   printf("Checking Object File.\n\r");
   check_objects(obj_f);

   printf("Checking Zone File .\n\r");
   check_zones(zon_f);

  printf("\n\r\nCheck successfully completed without any obvious errors.\n\r");

   fclose(zon_f);
   fclose(wld_f);
   fclose(mob_f);
   fclose(obj_f);
   fclose(newf);
}
