/* *************************************************************************
 *  file: shop.c , Shop module.                            Part of DIKUMUD *
 *  Usage: Procedures handling shops and shopkeepers.                      *
 *  Copyright (C) 1990, 1991 - see 'license.doc' for complete information. *
 *  All Rights Reserved                                                    *
 *  Using any part of DikuMud without having read license.doc is a         *
 *  violation of our copyright.                                            *
 ************************************************************************* */

#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "structs.h"
#include "utils.h"
#include "skills.h"
#include "comm.h"
#include "handler.h"
#include "db.h"
#include "interpreter.h"


#define SHOP_FILE "shop_info.shop"


struct shop_data
{
   struct file_index_type *keeper;/* Pointer to keeper                    */
   struct file_index_type **prod; /* Array of producing items, last nil   */
   sbit16 *type;                  /* Which item types to trade, last -1   */
   float  profit_buy;             /* Factor to multiply cost with.        */
   float  profit_sell;            /* Factor to multiply cost with.        */
   char *no_such_item1;           /* Message if keeper hasn't got an item */
   char *no_such_item2;           /* Message if player hasn't got an item */
   char *missing_cash1;           /* Message if keeper hasn't got cash    */
   char *missing_cash2;           /* Message if player hasn't got cash    */
   char *do_not_buy;              /* If keeper dosn't buy such things.    */
   char *message_buy;             /* Message when player buys item        */
   char *message_sell;            /* Message when player sells item       */
   int temper1;                   /* How does keeper react if no money    */
   int temper2;                   /* How does keeper react when attacked  */
   int with_who;                  /* Who does the shop trade with?        */
   int open1,open2;               /* When does the shop open?             */
   int close1,close2;             /* When does the shop close?            */
};



/* Global vars */
struct shop_data *shop_index = 0;
int number_of_shops = 0;


/* Externals */
extern struct time_info_data time_info;
char *fread_string(FILE *fl);



int is_ok(struct unit_data *keeper, struct unit_data *ch, int shop_nr)
{
   if (!CAN_SEE(keeper,ch))
   {
     act("$1n says, 'I don't trade with someone I can't see!'",
         FALSE, keeper,0,0,TO_ROOM);
     return FALSE;
   }

   if (shop_index[shop_nr].open1 > time_info.hours)
   {
      act("$1n says, 'Come back later, we're closed.'",
          FALSE, keeper,0,0,TO_ROOM);
      return FALSE;
   }
   else if (shop_index[shop_nr].close1 < time_info.hours)
   {
      if (shop_index[shop_nr].open2>time_info.hours)
      {
         act("$1n says, 'Sorry, we have closed, but come back later.'",
             FALSE,keeper,0,0, TO_ROOM);
         return FALSE;
      }
      else if (shop_index[shop_nr].close2<time_info.hours)
      {
         act("$1n says, 'We're closed, come back tomorrow.'",
             FALSE,keeper,0,0,TO_ROOM);
         return FALSE;
      }
   }

   switch(shop_index[shop_nr].with_who)
   {
     case 0 :
       return TRUE;
       break;

     case 1 :
       return TRUE;
       break;

     default :
       return TRUE;
       break;
   }
}



int trade_with(struct unit_data *item, int shop_nr)
{
   int counter;

   if (!IS_OBJ(item))
      return FALSE;

   if (OBJ_PRICE(item) < 1)
      return FALSE;

  for (counter=0; shop_index[shop_nr].type[counter] != -1; counter++)
    if (shop_index[shop_nr].type[counter] == OBJ_TYPE(item))
      return TRUE;

  return FALSE;
}



int shop_producing(struct unit_data *item, int shop_nr)
{
   int counter;

   if (!UNIT_FILE_INDEX(item))
      return FALSE;

   for (counter=0; shop_index[shop_nr].prod[counter]; counter++)
      if (shop_index[shop_nr].prod[counter] == UNIT_FILE_INDEX(item))
         return TRUE;

  return FALSE;
}



void shopping_buy( char *arg, struct unit_data *ch,
   struct unit_data *keeper, int shop_nr)
{
  char buf[MAX_STRING_LENGTH];
  struct unit_data *temp1;

  if (!is_ok(keeper,ch,shop_nr))
    return;
  
  if (str_is_empty(arg))
  {
    act("$1n tells you, 'what do you want to buy??'",
        FALSE, keeper,0,ch,TO_VICT);
    return;
  }

  if (!(temp1 = find_unit(keeper,&arg,0,FIND_UNIT_INVEN)) )
  {
     act(shop_index[shop_nr].no_such_item1, FALSE, keeper, 0, ch, TO_ROOM);
     return;
  }

  if (!IS_OBJ(temp1))
  {
     act("$1n says, 'I only trade objects!'", FALSE, keeper,0,ch,TO_ROOM);
     return;
  }

  if (OBJ_PRICE(temp1) <= 0)
  {
    act(shop_index[shop_nr].no_such_item1, FALSE, keeper, 0, ch, TO_ROOM);
    extract_unit(temp1);
    return;
  }


  if ((CHAR_GOLD(ch) < (int) (OBJ_PRICE(temp1)*
       shop_index[shop_nr].profit_buy)) && (CHAR_LEVEL(ch) < 200))
  {
     act(shop_index[shop_nr].missing_cash2, FALSE, keeper, temp1, ch, TO_ROOM);

     switch (shop_index[shop_nr].temper1)
     {
       case 0:
         do_action(keeper,UNIT_NAME(ch),30);
         return;
       case 1:
         act("$1n smokes on his joint.", FALSE, keeper,0,0,TO_ROOM);
         return;
       default:
         return;
     }
  }
  
  if ((CHAR_CARRYING_N(ch) + 1 > CHAR_CAN_CARRY_N(ch)))
  {
    act("$2n : You can't carry that many items.",
       FALSE,ch,temp1,0,TO_CHAR);
    return;
  }

  if ((CHAR_CARRYING_W(ch) + UNIT_WEIGHT(temp1)) > CHAR_CAN_CARRY_W(ch))
  {
    act("$2n : You can't carry that much weight.",
      FALSE,ch,temp1,0,TO_CHAR);
    return;
  }

  act("$1n buys $2n.", FALSE, ch, temp1, 0, TO_ROOM);

  sprintf(buf, shop_index[shop_nr].message_buy,
    (int) (OBJ_PRICE(temp1)*shop_index[shop_nr].profit_buy));
  act(buf, FALSE, keeper, temp1, ch, TO_VICT);

  act("You now have $2n.",FALSE, ch,temp1,0,TO_CHAR);

  if (CHAR_LEVEL(ch) < 220)
    CHAR_GOLD(ch) -= (int)(OBJ_PRICE(temp1)*shop_index[shop_nr].profit_buy);

  CHAR_GOLD(keeper) += (int) (OBJ_PRICE(temp1)*shop_index[shop_nr].profit_buy);

  /* Test if producing shop ! */
  if (shop_producing(temp1,shop_nr))
    temp1 = read_unit(UNIT_FILE_INDEX(temp1));
  else
    unit_from_unit(temp1);

  unit_to_unit(temp1, ch);
}



void shopping_sell( char *arg, struct unit_data *ch,
   struct unit_data *keeper,int shop_nr)
{
  char buf[MAX_STRING_LENGTH];
  struct unit_data *temp1;
  char *tmparg;

  if (!is_ok(keeper,ch,shop_nr))
    return;

  if (str_is_empty(arg))
  {
    act("$1n says, 'What do you want to sell??'",
        FALSE, keeper, 0, ch, TO_ROOM);
    return;
  }

  tmparg = arg;
  if ( !(temp1 = find_unit(ch, &tmparg, 0, FIND_UNIT_INVEN)) )
  {
    act(shop_index[shop_nr].no_such_item2, FALSE, keeper, 0, ch, TO_ROOM);
    return;
  }

  if (!IS_OBJ(temp1))
    return;

  if (!trade_with(temp1,shop_nr) || (OBJ_PRICE(temp1) < 1))
  {
    act(shop_index[shop_nr].do_not_buy, FALSE, keeper,temp1, ch, TO_ROOM);
    return;
  }

  if (CHAR_GOLD(keeper)< (int) (OBJ_PRICE(temp1)*
    shop_index[shop_nr].profit_sell))
  {
    act(shop_index[shop_nr].missing_cash1, FALSE, keeper,temp1, ch, TO_ROOM);
    return;
  }

  act("$1n sells $2n to $3n.", FALSE, ch, temp1, keeper, TO_ROOM);

  sprintf(buf,shop_index[shop_nr].message_sell,
       (int) (OBJ_PRICE(temp1)*shop_index[shop_nr].profit_sell));
  act(buf, FALSE, keeper,temp1, ch, TO_ROOM);

  act("$1n now has $2n.", FALSE, keeper, temp1, ch, TO_VICT);

  CHAR_GOLD(ch) += (int) (OBJ_PRICE(temp1)*shop_index[shop_nr].profit_sell);
  CHAR_GOLD(keeper) -= (int) (OBJ_PRICE(temp1)*shop_index[shop_nr].profit_sell);

  tmparg=arg;
  if (find_unit(keeper,&tmparg,0,FIND_UNIT_INVEN) ||
     (OBJ_TYPE(temp1) == ITEM_TRASH))
    extract_unit(temp1);
  else
  {
    unit_up(temp1);
    unit_down(temp1,keeper);
  }
}



void shopping_value( char *arg, struct unit_data *ch, 
  struct unit_data *keeper, int shop_nr)
{
  char buf[MAX_STRING_LENGTH];
  struct unit_data *temp1;

  if (!is_ok(keeper,ch,shop_nr))
    return;

  if (str_is_empty(arg))
  {
    act("$1n says, 'What do you want me to value??'",
        FALSE, keeper, 0, ch, TO_ROOM);
    return;
  }

  if (!(temp1 = find_unit(ch,&arg, 0, FIND_UNIT_EQUIP | FIND_UNIT_INVEN)))
  {
    act(shop_index[shop_nr].no_such_item2, FALSE, keeper,temp1, ch, TO_ROOM);
    return;
  }

  if (!trade_with(temp1,shop_nr))
  {
    act(shop_index[shop_nr].do_not_buy,FALSE, keeper,temp1, ch, TO_ROOM);
    return;
  }

  sprintf(buf,"$1n says, 'I'll give you %d gold coins for $2n!'",
     (int) (OBJ_PRICE(temp1)*shop_index[shop_nr].profit_sell));

  act(buf, FALSE, keeper, temp1, ch, TO_ROOM);
}



void shopping_list( char *arg, struct unit_data *ch,
   struct unit_data *keeper, int shop_nr)
{
  char buf[MAX_STRING_LENGTH], buf2[100], buf3[100];
  struct unit_data *temp1;
  int found_obj;

  extern char *drinks[];

  if (!is_ok(keeper,ch,shop_nr))
    return;

  strcpy(buf,"You can buy:\n\r");
  found_obj = FALSE;

  for (temp1=UNIT_CONTAINS(keeper); temp1; temp1 = temp1->next)
    if (IS_OBJ(temp1) && !OBJ_EQP_POS(temp1) &&
        CAN_SEE(ch,temp1) && (OBJ_PRICE(temp1)>0))
    {
      found_obj = TRUE;
      if (OBJ_TYPE(temp1) != ITEM_DRINKCON) 
      {
        sprintf(buf2,"%s for %d gold coins.\n\r",
          UNIT_TITLE(temp1),
          (int) (OBJ_PRICE(temp1)*shop_index[shop_nr].profit_buy));
      }
      else
      {
        if (OBJ_VALUE(temp1, 1))
          sprintf(buf3,"%s of %s", UNIT_TITLE(temp1),
            drinks[OBJ_VALUE(temp1, 2)]);
        else
          sprintf(buf3,"%s",UNIT_TITLE(temp1));

        sprintf(buf2,"%s for %d gold coins.\n\r",buf3,
        (int)(OBJ_PRICE(temp1)*shop_index[shop_nr].profit_buy));
      }

      strcat(buf,CAP(buf2));
    }

  if (!found_obj)
    strcat(buf,"Nothing!\n\r");

  send_to_char(buf,ch);
}



void shopping_kill(char *arg, struct unit_data *ch,
  struct unit_data *keeper, int shop_nr)
{
  switch (shop_index[shop_nr].temper2)
  {
    case 0:
      act("$1n growls, 'Don't ever try that again!",
          FALSE, keeper, 0, ch, TO_ROOM);
      return;

    case 1:
      act("$1n says, 'Scram - midget!'",
        FALSE, keeper, 0, ch, TO_ROOM);
      return;

    default :
      return;
  }
}



/* Special routine for all the shopkeepers */

int shop_keeper(struct unit_data *keeper, struct unit_data *ch,
                struct command_info *cmd, struct unit_fptr *fptr,
                char *arg)
{
  int shop_nr;

  /* Figure out which shop-data this character needs */
  for (shop_nr=0; (shop_nr < number_of_shops) &&
       (shop_index[shop_nr].keeper != UNIT_FILE_INDEX(keeper));
       shop_nr++);

  if (shop_nr >= number_of_shops)
  {
     log("SHOP ERROR: Shop keeper not located.");
     return FALSE;
  }

  if (cmd->no == 56)  /* Buy */
  {
    shopping_buy(arg,ch,keeper,shop_nr);
    return TRUE;
  }
  else if (cmd->no ==57) /* Sell */
  {
    shopping_sell(arg,ch,keeper,shop_nr);
    return TRUE;
  }
  else if (cmd->no == 58) /* value */
  {
    shopping_value(arg,ch,keeper,shop_nr);
    return TRUE;
  }
  else if (cmd->no == 59) /* List */
  {
    shopping_list(arg,ch,keeper,shop_nr);
    return TRUE;
  }

  return FALSE;
}





/*
  Format of the shop file is approximately:

  Shop file ::= keeper profit profit newline {prod} end types
                msg msg msg msg msg msg msg
                temper temper newline
                race open open
  Keeper    ::= Prod
  Prod      ::= (zone/name | name@zone) '~' newline
  Profit    ::= <floating point number>
  Msg       ::= <string> '~' newline
  Temper    ::= <integer>
  Race      ::= <integer> newline
  Open      ::= <hour> '..' <hour> newline
  End       ::= '$~' newline
  Types     ::= { <item type> } -1 newline
*/

void boot_the_shops(void)
{
   char *buf;
   int temp;
   int count;
   FILE *shop_f;
   struct file_index_type *fi;

   if (!(shop_f = fopen(SHOP_FILE, "r")))
   {
     log("Error in boot shop, could not open file:");
     log(SHOP_FILE);
     exit(0);
   }

   for (number_of_shops = 0;
       !feof(shop_f) && (buf = fread_string(shop_f)); number_of_shops++)
   {
      if (number_of_shops == 0)  /* first shop */
         CREATE(shop_index, struct shop_data, 1);
      else
         RECREATE(shop_index, struct shop_data, number_of_shops+1);

      /* Assign the keeper */
      shop_index[number_of_shops].keeper = str_to_file_index(buf);
      free(buf);

      /* Read profits */
      fscanf(shop_f,"%f %f \n",
         &shop_index[number_of_shops].profit_buy,
         &shop_index[number_of_shops].profit_sell);

      /* Read productions */
      CREATE(shop_index[number_of_shops].prod, struct file_index_type *, 1);

      shop_index[number_of_shops].prod[0] = 0;

      for(count = 0; (buf = fread_string(shop_f)) && (*buf != '$'); count++)
      {
         RECREATE(shop_index[number_of_shops].prod,
		  struct file_index_type *, count+2);
         shop_index[number_of_shops].prod[count+1] = 0;
         shop_index[number_of_shops].prod[count] =
            str_to_file_index(buf);
         free(buf);
      }
      if (buf)
         free(buf);


      /* Read item types that are traded */
      fscanf(shop_f,"%d ", &temp);
      CREATE(shop_index[number_of_shops].type, sbit16, 1);
      shop_index[number_of_shops].type[0] = -1;

      for (count=0; temp != -1; count++)
      {
         RECREATE(shop_index[number_of_shops].type, sbit16, count+2);
         shop_index[number_of_shops].type[count] = (sbit16) temp;
         shop_index[number_of_shops].type[count+1] = -1;
         fscanf(shop_f,"%d ", &temp);
      }

      /* Read seven messages */
      shop_index[number_of_shops].no_such_item1 =
         fread_string(shop_f);
      shop_index[number_of_shops].no_such_item2 =
        fread_string(shop_f);
      shop_index[number_of_shops].do_not_buy =
        fread_string(shop_f);
      shop_index[number_of_shops].missing_cash1 =
        fread_string(shop_f);
      shop_index[number_of_shops].missing_cash2 =
        fread_string(shop_f);
       shop_index[number_of_shops].message_buy =
         fread_string(shop_f);
       shop_index[number_of_shops].message_sell =
         fread_string(shop_f);

       /* Read temper */
       fscanf(shop_f,"%d \n",
         &shop_index[number_of_shops].temper1);
       fscanf(shop_f,"%d \n",
         &shop_index[number_of_shops].temper2);

       /* Read race that keeper trades with */
       /* Ought to be expanded              */
       fscanf(shop_f,"%d \n",
         &shop_index[number_of_shops].with_who);

       /* Read 2*opening times */
       fscanf(shop_f,"%d..%d \n",
         &shop_index[number_of_shops].open1,
         &shop_index[number_of_shops].close1);

       fscanf(shop_f,"%d..%d \n",
         &shop_index[number_of_shops].open2,
         &shop_index[number_of_shops].close2);
   } /* FOR */

   if (buf)
      free(buf);

   fclose(shop_f);
}
